
package Controls;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/** Eine Hilfs-Klasse, in der verschiedene, nuetzliche Klassen-Methoden implementiert sind. */

public final class Tools implements Ctrl {

        /**
        * Don't let anyone instantiate this class.
        */
        private Tools() {}


	/**
	 * Neue Platz-Kalkulation fuer die gesamte Applikation.
	 */

	public static void validateApp(Component comp) {

		Component parent = comp;

		while (! (parent instanceof Frame)) parent = parent.getParent();

		parent.validate();

		((Frame) parent).pack();
	}

	/**
	 * Ergaenzen eines Strings zu einem Filenamen.
	 *
	 * @param name File-Name (oder Teil desselben).
	 * @param mode Modus der Operation (LOAD oder SAVE).
	 * @param comp Komponente, die die Anfrage aufruft.
	 *
	 * @return Vollstaendiger Filename als String (null -> kein Name ausgewaehlt).
	 */

	public static String completeFilename(String name, int mode, Component comp) {

		if (name != null) {

			File file = new File(name);

			if (file.isFile()) {

				return(name);

			} else if (! file.isDirectory()) {

				System.err.println("Tools.completeFilename:: File-Error: " + name + " is neither file nor directory");
				return(null);
			}
		}

		Component parent = comp;

		while (! (parent instanceof Frame)) parent = parent.getParent();

		FileDialog dialog = new FileDialog((Frame) parent, mode == SAVE ? "Save" : "Load", mode);

		if (name != null ) {

			dialog.setDirectory(name);
		}

		dialog.show();

		if (dialog.getFile() == null) {

			return(null);

		} else {

			return(dialog.getDirectory() + dialog.getFile());
		}
	}

	/**
	 * Laden eines Image aus einem File.
	 * Es wird versucht, den Inhalt des Files im vorgegebenen Format zu laden.
	 *
	 * @param fname File-Name.
	 * @param format File-Format des Files.
	 * @param comp Komponente, die die Anfrage aufruft.
	 *
	 * @return Geladenes Bild als Image (null -> Fehler beim Laden).
	 */

	public static Image loadImage(String fname, int typ, Component comp) {

		Image img = null;

		switch (typ) {

			case STD:
				img = load_std(fname, comp);
				break;

			case PPM:
				img = load_ppm(fname, comp);
				break;

			default:
				System.err.println("Tools.loadImage:: Error: Unsupported File-Format: " + typ);
		}

		return (img);
	}

	/**
	 * Laden eines Image aus einem File.
	 * Es wird versucht, den Inhalt des Files in einem der bekannten Formate zu laden.
	 *
	 * @param fname File-Name.
	 * @param comp Komponente, die die Anfrage aufruft.
	 *
	 * @return Geladenes Bild als Image (null -> Fehler beim Laden).
	 */

	public static Image loadImage(String fname, Component comp) {

		Image img = loadImage(fname, STD, comp);

		if (img == null) {

			img = loadImage(fname, PPM, comp);
		}

		return (img);
	}

		
	private static Image load_std(String fname, Component comp) {		// Versuch, fname als Java-Standard-Format zu laden

		Image img = comp.getToolkit().getImage(fname);

		MediaTracker track = new MediaTracker(comp);

		try {
			track.addImage(img, 0);
			track.waitForID(0);

		} catch (InterruptedException ex) {

			System.err.println("Tools.load_std:: Load-Interrupted: " + ex);
			return (null);
		}

		if ((track.statusID(0, false) & MediaTracker.ERRORED) != 0) {

			return (null);
		}

		return (img);
	}

	private static Image load_ppm(String fname, Component comp) {		// Versuch, fname als (spezielles) PPM zu laden

		Image img = null;

		try {
			DataInputStream in = new DataInputStream(new BufferedInputStream(new FileInputStream(fname)));

			if (! getLine(in).equals("P6")) {

				return(null);
			}

			String line = getLine(in);

			int ix = line.indexOf(' ');

			int w = Integer.parseInt(line.substring(0, ix));
			int h = Integer.parseInt(line.substring(ix+1));

			getLine(in);		// Max. Color No. weglesen
				
			int len = w * h;

			int[] pixels = new int[len];

			int r, g, b;

			for (int i=0; i<len; i++) {

				r = in.readUnsignedByte();
				g = in.readUnsignedByte();
				b = in.readUnsignedByte();

				pixels[i] = (255 << 24) + (r << 16) + (g << 8) + b; 
			}

			in.close();

			img = comp.createImage(new MemoryImageSource(w, h, pixels, 0, w));

		} catch (IOException ex) {

			System.err.println("Tools.load_ppm:: IO-Error: " + ex);
			return(null);

		} catch (NumberFormatException ex) {

			System.err.println("Tools.load_ppm:: PPM-Format-Error: " + ex);
			return(null);
		}

		MediaTracker track = new MediaTracker(comp);

		try {
			track.addImage(img, 0);
			track.waitForID(0);

		} catch (InterruptedException ex) {

			System.err.println("Tools.load_ppm:: Load-Interrupted: " + ex);
			return (null);
		}

		if ((track.statusID(0, false) & MediaTracker.ERRORED) != 0) {

			return (null);
		}

		return(img);
	}

	private static String getLine(DataInputStream in) throws IOException {

		String line = "";
		char ch;

		while ((ch = (char) in.readByte()) != '\n') {

			line += ch;
		}

		return line;
	}
}
