
package Controls;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;

/** Ein Control-Objekt zur Ausgabe von grafischen Operationen. */

public class MalFeld extends CtrlBox implements MouseListener, MouseMotionListener {

	private SBCanvas sbc;
	private DBCanvas canvas;

	private int dimX;
	private int dimY;

	private Color malFarbe;
	private Color feldFarbe;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein MalFeld-Objekt mit Name, Groessen, Dimensionen, Callbacks und Farben.
	 *
	 * @param name	Name der Box.
	 * @param sizeX	Horizontale Groesse des MalFeld (sichtbarer Bereich).
	 * @param sizeY	Vertikale Groesse des MalFeld (sichtbarer Bereich).
	 * @param dimX	Horizontale Dimension des Zeichen-Feldes (gesamter Bereich).
	 * @param dimY	Vertikale Dimension des Zeichen-Feldes (gesamter Bereich).
	 * @param cb	Master-Objekt fuer Callback.
	 * @param id	Master-ID fuer Callback.
	 * @param idc	Master-ID fuer Canvas-Callback.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param malFarbe  Vordergrund-Farbe des Zeichen-Feldes.
	 * @param feldFarbe Hintergrund-Farbe des Zeichen-Feldes.
	 */

	public MalFeld(String name, int sizeX, int sizeY, int dimX, int dimY, CB cb, int id, int idc, Color fg, Color bg, Color malFarbe, Color feldFarbe) {

		super(name, RAND, fg, bg);

		this.cb = cb;
		this.id = id;

		this.dimX = dimX;
		this.dimY = dimY;

		this.malFarbe  = malFarbe;
		this.feldFarbe = feldFarbe;

		sbc = new SBCanvas(cb, idc, sizeX, sizeY, dimX, dimY);

		addBag(sbc, 0, 0, 1, 1);

		canvas = sbc.getCanvas();

		canvas.setForeground(malFarbe);
		canvas.setBackground(feldFarbe);

		if (cb != null) canvas.addMouseListener(this);
		if (cb != null) canvas.addMouseMotionListener(this);
	}

	/** Erzeugt ein MalFeld-Objekt mit Default-Farben. */

	public MalFeld(String name, int sizeX, int sizeY, int dimX, int dimY, CB cb, int id, int idc) {

		this(name, sizeX, sizeY, dimX, dimY, cb, id, idc, null, null, Color.black, Color.white);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback fuer Maus-Click wird aufgerufen.
	 * Argument: zugehoerige Maus-Aktion.
	 */

	public void mouseClicked(MouseEvent e) {

		cb.callback(id, this, new Maus(Maus.CLICK, e.getPoint()));
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback fuer Maus-Enter wird aufgerufen.
	 * Argument: zugehoerige Maus-Aktion.
	 */

	public void mouseEntered(MouseEvent e) {

		cb.callback(id, this, new Maus(Maus.ENTER));
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback fuer Maus-Exit wird aufgerufen.
	 * Argument: zugehoerige Maus-Aktion.
	 */

	public void mouseExited(MouseEvent e) {

		cb.callback(id, this, new Maus(Maus.EXIT));
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback fuer Maus-Press wird aufgerufen.
	 * Argument: zugehoerige Maus-Aktion.
	 */

	public void mousePressed(MouseEvent e) {

		cb.callback(id, this, new Maus(Maus.PRESS, e.getPoint()));
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback fuer Maus-Release wird aufgerufen.
	 * Argument: zugehoerige Maus-Aktion.
	 */

	public void mouseReleased(MouseEvent e) {

		cb.callback(id, this, new Maus(Maus.RELEASE, e.getPoint()));
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback fuer Maus-Drag wird aufgerufen.
	 * Argument: zugehoerige Maus-Aktion.
	 */

	public void mouseDragged(MouseEvent e) {

		cb.callback(id, this, new Maus(Maus.DRAG, e.getPoint()));
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback fuer Maus-Move wird aufgerufen.
	 * Argument: zugehoerige Maus-Aktion.
	 */

	public void mouseMoved(MouseEvent e) {

		cb.callback(id, this, new Maus(Maus.MOVE, e.getPoint()));
	}

	/**
	 * Liefert das Graphics-Objekt Screen.
	 *
	 * @return Gibt das Graphics-Objekt fuer ungebuffertes Zeichnen zurueck.
	 */

	public Graphics getGScreen() {

		return(canvas.getGraphics());
	}

	/**
	 * Liefert das Graphics-Objekt Buffer.
	 *
	 * @return Gibt das Graphics-Objekt fuer gebuffertes Zeichnen zurueck.
	 */

	public Graphics getGBuffer() {

		return(canvas.getGBuffer());
	}

	/**
	 * Aktiviert die Ausgabe des Double-Buffers.
	 *
	 * Ein MalFeld enthaelt zwei Objekte, in die gezeichnet werden kann:
	 * Das Screen-Objekt gibt alle Operationen sofort direkt auf dem Bildschirm aus, ohne sie zu speichern.
	 * Das Buffer-Objekt fuehrt alle Operationen in einem Memory-Buffer aus. Dieser Buffer wird bei jeder
	 * Refresh-Operation auf den Schirm kopiert oder direkt mit dieser Methode.
	 */

	public void showBuffer() {

		canvas.showBuffer();
	}

	/**
	 * Liefert die Zeichen-Farbe.
	 *
	 * @return Gibt die Default-Vordergrund-Farbe des Zeichen-Feldes zurueck.
	 */

	public Color getMalFarbe() {

		return(malFarbe);
	}

	/**
	 * Liefert die Feld-Farbe.
	 *
	 * @return Gibt die Hintergrund-Farbe des Zeichen-Feldes zurueck.
	 */

	public Color getFeldFarbe() {

		return(feldFarbe);
	}

	/**
	 * Aendert die Farben des MalFeld-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 */

	public void chgColors(Color fg, Color bg) {

		chgBoxColors(fg, bg);
	}

	/**
	 * Reset des MalFeld.
	 *
	 * Ersetzt alle internen Objekte durch neue mit geaenderten Parametern.
	 * @param dimX	Neue horizontale Dimension des Zeichen-Feldes.
	 * @param dimY	Neue vertikale Dimension des Zeichen-Feldes.
	 * @param malFarbe  Neue Vordergrund-Farbe des Zeichen-Feldes.
	 * @param feldFarbe Neue Hintergrund-Farbe des Zeichen-Feldes.
	 */

	public void resetCanvas(int dimX, int dimY, Color malFarbe, Color feldFarbe) {

		this.dimX = dimX;
		this.dimY = dimY;

		this.malFarbe  = malFarbe;
		this.feldFarbe = feldFarbe;

		sbc.resetCanvas(dimX, dimY, malFarbe, feldFarbe);
	}

	/**
	 * Einlesen eines Bildes mit teilweiser Vorgabe des Filenamen.
	 *
	 * @param name File-Name (oder Teil desselben).
	 */

	public void load(String name) {

		String fname = Tools.completeFilename(name, LOAD, this);

		if (fname == null) return;

		Image img = Tools.loadImage(fname, this);

		if (img == null) {

			System.err.println("MalFeld.load:: Load-Error");
			return;
		}

		int w = img.getWidth(null);
		int h = img.getHeight(null);

		resetCanvas(w, h, malFarbe, feldFarbe);

		getGBuffer().drawImage(img, 0, 0, null);
		showBuffer();
	}

	/** Einlesen eines Bildes mit interaktiver Bestimmung des Filenamen. */

	public void load() {

		load(null);
	}

	/**
	 * Speichern eines Bildausschnitts mit teilweiser Vorgabe des Filenamen.
	 *
	 * @param name File-Name (oder Teil desselben), unter dem der Ausschnitt abgespeichert werden soll.
	 * @param x X-Koordinate der linken, oberen Ecke des Bildausschnitts.
	 * @param y Y-Koordinate der linken, oberen Ecke des Bildausschnitts.
	 * @param w Breite des Bildausschnitts.
	 * @param h Hoehe des Bildausschnitts.
	 */

	public void save(String name, int x, int y, int w, int h) {

		String fname = Tools.completeFilename(name, SAVE, this);

		if (fname == null) return;

		try {
			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(fname)));

			out.writeBytes("P6\n");
			out.writeBytes(w + " " + h +"\n");
			out.writeBytes("255\n");

			int[] pixels = new int[w * h];

			PixelGrabber grabber = canvas.getPixelGrabber(x, y, w, h, pixels);

			if(!grabber.grabPixels()) {

				System.err.println("MalFeld.save:: Can't grab Pixels!");
				out.close();
				return;
			}

			int len = w * h;

			for(int i=0; i < len; i++) {

				out.writeByte((pixels[i] >> 16) & 0xff);
				out.writeByte((pixels[i] >>  8) & 0xff);
				out.writeByte( pixels[i]        & 0xff);
			}
		
			out.close();

		} catch (InterruptedException ex) {

			System.err.println("MalFeld.save:: Grab-Error: " + ex);

		} catch (IOException ex) {

			System.err.println("MalFeld.save:: IO-Error: " + ex);
		}
	}

	/** Speichern eines Bildausschnitts mit interaktiver Bestimmung des Filenamen. */

	public void save(int x, int y, int w, int h) {

		save(null, x, y, w, h);
	}

	/** Speichern des gesamten Bilds mit teilweiser Vorgabe des Filenamen. */

	public void save(String fname) {

		save(fname, 0, 0, dimX, dimY);
	}

	/** Speichern des gesamten Bilds mit interaktiver Bestimmung des Filenamen. */

	public void save() {

		save(null, 0, 0, dimX, dimY);
	}
}
