
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Auswahl eines Wertes aus einem festen Bereich. */

public class Regler extends CtrlBox implements AdjustmentListener {

	private Scrollbar slider;
	private Label     valueLabel;

	private int orient;

	private int val;
	private int min;
	private int max;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein Regler-Objekt mit Name, Orientierung, Werten, Callback, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param orient Orientierung des Regler-Objekts.
	 * @param min	Kleinster Wert.
	 * @param max	Groesster Wert.
	 * @param init	Initialer Wert.
	 * @param cb	Master-Objekt fuer Callback (null -> kein EventListener).
	 * @param id	Master-ID fuer Callback.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Regler(String name, int orient, int min, int max, int init, CB cb, int id, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		if (min > max) {int tmp = min; min = max; max = tmp;}
		if (init < min) init = min;
		if (init > max) init = max;

		this.orient = orient;

		this.min = min;
		this.max = max;
		this.val = init;

		this.cb = cb;
		this.id = id;

		if (orient == HORZ) {

			valueLabel = new Label(init + "", Label.CENTER);

			slider = new Scrollbar(Scrollbar.HORIZONTAL, init - min, 1, 0, 1 + max - min);

			addBag(new Label(name + ":", Label.CENTER),
				0, 0, 1, 1, E, PAD, PAD, PAD, 0);
			addBag(valueLabel,
				1, 0, 1, 1, C, PAD, PAD, 0,   0);
			addBag(slider,
				2, 0, 1, 1, W, PAD, PAD, 0, PAD);

		} else {	// orient == VERT, oder wie auch immer

			valueLabel = new Label(init + "", Label.CENTER);

			slider = new Scrollbar(Scrollbar.VERTICAL, max - init, 1, 0, 1 + max - min);

			addBag(valueLabel,
				0, 0, 1, 1, N, PAD, 0, PAD, PAD);
			addBag(slider,
				0, 1, 1, 1, C, 0,   0, PAD, PAD);
			addBag(new Label(name, Label.CENTER),
				0, 2, 1, 1, S, 0, PAD, PAD, PAD);
		}

		slider.setForeground(sfg == null ? FG : sfg);
		slider.setBackground(sbg == null ? BG : sbg);

		slider.addAdjustmentListener(this);
	}

	/** Erzeugt ein Regler-Objekt mit Default-Farben. */

	public Regler(String name, int orient, int min, int max, int init, CB cb, int id, boolean rand) {

		this(name, orient, min, max, init, cb, id, rand, null, null, null, null);
	}

	/** Erzeugt ein Regler-Objekt mit Rand. */

	public Regler(String name, int orient, int min, int max, int init, CB cb, int id, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, orient, min, max, init, cb, id, true, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Regler-Objekt mit Rand und Default-Farben. */

	public Regler(String name, int orient, int min, int max, int init, CB cb, int id) {

		this(name, orient, min, max, init, cb, id, true, null, null, null, null);
	}

	/** Erzeugt ein Regler-Objekt mit Rand und Default-Farben ohne Callback. */

	public Regler(String name, int orient, int min, int max, int init) {

		this(name, orient, min, max, init, null, 0, true, null, null, null, null);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Argument: Ausgewaehlter Wert.
	 */

	public void adjustmentValueChanged(AdjustmentEvent e) {

		int v = e.getValue();

		if (orient == HORZ) {

			v += min;

		} else {

			v = max - v;
		}

		if (v != val) {

			val = v;

			valueLabel.setText(val + "");

			if (cb != null) cb.callback(id, this, new Integer(val));
		}
	}

	/**
	 * Setzt den Wert.
	 *
	 * @param item Neuer aktueller Wert.
	 */

	public void setValue(int v) {

		if (v < min) v = min;
		if (v > max) v = max;

		val = v;

		if (orient == HORZ) {

			slider.setValue(val - min);

		} else {

			slider.setValue(max - val);
		}

		valueLabel.setText(val + "");
	}

	/**
	 * Liefert den Wert.
	 *
	 * @return Gibt den aktuellen Wert als int zurueck.
	 */

	public int getValue() {

		int v = slider.getValue();

		if (orient == HORZ) {

			v += min;

		} else {

			v = max - v;
		}

		return(v);
	}

	/**
	 * Aendert die Farben des Regler-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		slider.setForeground(sfg == null ? FG : sfg);
		slider.setBackground(sbg == null ? BG : sbg);
	}
}
